/* commands.c - gnapster command handling */

#include <stdio.h>
#include <stdlib.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>

#include <signal.h>

#include <gnome.h>

#include "napster.h"
#include "gnapster.h"
#include "ui.h"
#include "callbacks.h"

#include "upload.h"
#include "network.h"
#include "chan.h"
#include "text.h"
#include "xtext.h"
#include "utils.h"

#include "commands.h"
#include "regexp.h"

#include "debug.h"

extern ConsoleTab console_tab;
extern UserInfo user_info;
extern ConnInfo conn_info;

extern GList *ignore_list;
extern GList *signore_list;

#define CHECK_ARGS() if (!args) { invalid_args(command); return; }
#define INVALID_ARGS() { invalid_args(command); return; }

COMMAND cmds[] = {
   /* Regular commands */
   { "/help",		do_help,	"[command]", 		"Shows help for Gnapster commands" },
   { "/version",	do_version,	"", 			"Requests server information" },
   { "/topic:/t",	do_topic,	"<topic>",		"Changes the current channels topic" },
   { "/clear:/cls",	do_clear,	"",			"Clears the current tab" },
   { "/sv",		do_sv,		"",			"Displays your client version to the current tab" },
   { "/whois:/wi",	do_whois,	"[user]",		"Requests server whois information for the user" },
   { "/msg:/m",		do_msg,		"<user> <message>", 	"Send a private message to another user" },
   { "/me",		do_me,		"<action>",		"Sends an action style message to the current channel" },
   { "/query:/q",	do_query, 	"[user]",		"Insert another tab with a private conversation to a user in it" },
   { "/ignore:/ig",	do_ignore,	"[user]", 		"Place the user on your ignore list" },
   { "/unignore:/uig",	do_unignore,	"<user>",		"Removes a user from ignore" },
   { "/join:/j",	do_join,	"<channel>",		"Joins a Napster channel" },
   { "/part:/p",	do_part,	"",			"Leaves a Napster channel" },
   { "/raw",		do_raw,		"<message number> [message]",	"For developer use only!" },
   { "/share",		do_share,	"<add/remove> <filename>",	"Attempts to either add/remove the specified file (non-mp3) as a share.  Removing filenames requires any given string to exist in the file entry" },
   { "/stats",		do_stats, 	"",			"Displays server stats for opennap servers" },
   
   /* Admin Only commands 
    * please don't try to use these on official servers, I wrote them because
    * I am a moderator on opennap now...you probably aren't, so ignore them */
   { "/wallop:/wall",	do_wallop,	"<message>", 		"Send a wallop message to all admins/moderators" },
   { "/global",		do_global,	"<message",		"Send a global message to all users" },
   { "/op",		do_op,		"<user>",		"Sets the specified user as operator of the current channel" },
   { "/deop:/dop", 	do_deop,	"<user>",		"Removes the specified user as an operator of the current channel" },
   { "/kill",		do_kill,	"<user>",		"Disconnects a user with a lower status than you" },
   { "/kick:/k",	do_kick,	"<user> [reason]",	"Kicks the user from the current channel" },
   { "/cban",		do_cban,	"<user|ip> [reason]",	"Bans the user from the current channel" },
   { "/cunban", 	do_cunban,	"<user|ip>",		"Unbans the user from the current channel" },
   { "/cbans",		do_cbans,	"",			"Lists the current channel bans" },
   { "/cunban_all",	do_cunban_all,	"",			"Removes all channel bans on the current channel" },
   { "/ban",		do_ban,		"<user|ip> [reason]",	"Attempts to place a server ban on the user" },
   { "/unban",		do_unban,	"<user|ip>",		"Requests that the user be unbanned" },
   { "/bans",           do_bans,        "",                     "Shows the server ban list" },
   { "/muzzle",		do_muzzle,	"<user> [reason]",	"Removes a users ability to send public messages" },
   { "/unmuzzle",	do_unmuzzle,	"<user>",		"Restores a users ability to send public messages" },
   { "/mode",		do_mode,	"<channel> [mode]",	"Sets a channels mode" },
   { "/setport",	do_setport,	"<user> <port>",	"Attempts to change a users data port" },
   { "/userlevel:/ul",	do_userlevel,	"<user> <level>",	"Changes a users status level" },
   { "/setline",	do_setline,	"<user> <speed>",	"Changes a users line speed" },
   { "/links",		do_links,	"",			"Shows opennap server links" },
   { "/signore:/sig",	do_signore,	"<match string>",	"Ignores server messages that match the string (wildcards accepted)" },
   { "/unsignore:/usig",do_unsignore,	"<index>",		"Removes a server ignore from the list (to get the index number type /signore without arguments" },
   { "/sigsave",	do_sigsave,	"",			"Saves the server ignore list to disk and reloads when Gnapster runs again" },
   { "/chanlevel",	do_chanlevel, 	"<channel> <level>",	"Sets the minimum user level required to join the channel" },
   { "/usermode:/um",	do_usermode,	"[modes]",		"Sets your usermode which controls what type of messages you will see from the server" },
   { NULL,		NULL,		NULL },
};

void invalid_args(char *command) {
   gnapster_text_insert(CURR, SYSTEM,
			"Invalid arguments for \2%s\1.  Check \2/help\1\n",
			command);
}

int handle_command(char *command, char *args) {
   int i = 0, x;
   char *ptr, *hld, *cmd[2];
   
   while(cmds[i].command) {
      ptr = d_strdup(cmds[i].command);
      x = 0;
      
      while((cmd[x] = next_arg_full(ptr, &ptr, ':'))) {
	 if (!g_strcasecmp(command, cmd[x++])) {
	    x = -1;
	    break;
	 }
      }
      
      d_free(cmd[0]);
      if (x == -1) 
	break;
      i++;
   }

   if (cmds[i].func == NULL)
     return 0;
   
   if (!command)
     return 0;
   
   hld = d_strdup(args);
   
   cmds[i].func (command, args);
   
   d_free(hld);
   
   return 1;
}

void dispatch_text(ChannelInfo *chan, int page_num, char *text) {
   gnapster_text_insert(page_num, MESSAGE, "<\2%s\1> %s\n",
			user_info.user, text);
   napster_send(conn_info.sock,
		(chan->user) ? NAPSTER_USER_MESSAGE : NAPSTER_CHAN_MESSAGE,
		"%s %s", chan->channel_name, text);
}

GNAPSTER_COMMAND(do_sendtext) {
   ChannelInfo *chan;
   char *ptr, buf[181], *nptr;
   int page_num, i, last_space = 0, buf_pos = 0;
   
   page_num = gnapster_get_pagenum();
   if (page_num <= 1) return;
   
   chan = g_list_nth_data(console_tab.channel_list, 
			  page_num - 2);
   
   ptr = args;
   
   if (!chan->user)
     nptr = nick_completion(ptr, GTK_CLIST(chan->clist));
   else
     nptr = d_strdup(ptr);
   
   for(i=0; nptr[i]; i++) {
      if (nptr[i] == ' ')
	last_space = buf_pos;
      if (nptr[i] == '\n') {
	 buf[buf_pos] = 0;
	 buf_pos = 0;

	 dispatch_text(chan, page_num, buf);
      } else if (buf_pos > 170) {
	 i -= (buf_pos - last_space);
	 buf[last_space] = 0;
	 buf_pos = 0;

	 dispatch_text(chan, page_num, buf);
      } else
	buf[buf_pos++] = nptr[i];
   }
   
   if (buf_pos)
     buf[buf_pos] = 0, dispatch_text(chan, page_num, buf);
   
   d_free(nptr);
   d_free(args);
}

GNAPSTER_COMMAND(do_help) {
   int i = 0, x;
   char *slasharg, *ptr, *hld, *cmd[2];
   
   if (!args) {
      int page_num =
	gnapster_get_pagenum();
      
      i++;
      gnapster_text_insert(page_num, SYSTEM, "Command Listing:\n");
      while(cmds[i].command) {
	 ptr = d_strdup(cmds[i].command);
	 hld = ptr;
	 gnapster_text_insert(page_num, MESSAGE, "%c \2%s\1%c",
			      (i % 3 == 1)  ? '*' : ' ',
			      next_arg_full(hld, &hld, ':'), (i % 3 == 0) ? '\n' : ',');
	 d_free(ptr);
	 i++;
      }
      gnapster_text_insert(page_num, MESSAGE, "\n");
      gnapster_text_insert(page_num, SYSTEM, "For additional help use \2/help command\1\n");
      return;
   }
   
   slasharg = d_strdup_printf(DEBUG "/%s", next_arg(args, &args));
   while(cmds[i].command) {
      ptr = d_strdup(cmds[i].command);
      hld = ptr;
      x = 0;
      while((cmd[x] = next_arg_full(hld, &hld, ':'))) {
	 if (!g_strcasecmp(slasharg, cmd[x++])) {
	    x = -1;
	    break;
	 }
      }
      d_free(ptr);
      if (x == -1) 
	break;
      i++;
   }

   if (cmds[i].func == NULL) {
      d_free(slasharg);
      return;
   }

   gnapster_text_insert(CURR, SYSTEM,
			"\2%s %s\1 - %s\n",
			slasharg, cmds[i].syntax, cmds[i].help);
   
   d_free(slasharg);
}

GNAPSTER_COMMAND(do_version) {
   napster_send(conn_info.sock, NAPSTER_SERVER_VERSION, NULL);
}

GNAPSTER_COMMAND(do_wallop) {
   CHECK_ARGS();
   
   gnapster_text_insert(CURR, MESSAGE,
			"<*\2%s\1> %s\n", user_info.user, 
			args);
   napster_send(conn_info.sock, NAPSTER_ADMIN_MESSAGE, "%s",
		args);
}

GNAPSTER_COMMAND(do_global) {
   CHECK_ARGS();
   
   gnapster_text_insert(CURR, MESSAGE,
			"<\2%s\1/\2global\1> %s\n", user_info.user,
			args);
   napster_send(conn_info.sock, NAPSTER_GLOBAL_MESSAGE, "%s",
		args);
}

GNAPSTER_COMMAND(do_topic) {
   char *chan;

   CHECK_ARGS();
   
   chan = get_current_chan();
   if (!chan) 
     return;
   
   napster_send(conn_info.sock, NAPSTER_CHAN_TOPIC, "%s %s",
		chan, args);
}

GNAPSTER_COMMAND(do_clear) {
   ChannelInfo *chan;
   GtkWidget *text;
   
   chan = g_list_nth_data(console_tab.channel_list, 
			  gnapster_get_pagenum() - 2);
   if (chan)
     text = chan->text;
   else 
     text = console_tab.text;
   
   gtk_xtext_remove_lines(GTK_XTEXT(text), -1, TRUE);
}

GNAPSTER_COMMAND(do_sv) {
   int page_num;
   ChannelInfo *chan;
   
   page_num = gnapster_get_pagenum();
   
   if (args) {
      char *user;
      
      user = next_arg(args, &args);
      gnapster_text_insert(CURR, MESSAGE, "-> \2[%s]\1 %s\n",
			   user, gnapster_sv());
      napster_send(conn_info.sock, NAPSTER_USER_MESSAGE, "%s %s",
		   user, gnapster_sv());
      return;
   }
   
   if (page_num <= 1) 
     return;
   
   chan = g_list_nth_data(console_tab.channel_list,
			  page_num - 2);
   if (!chan) 
     return;
   
   gnapster_text_insert(page_num, MESSAGE, "<\2%s\1> %s\n", user_info.user,
			gnapster_sv());
   napster_send(conn_info.sock,
		(chan->user) ? NAPSTER_USER_MESSAGE : NAPSTER_CHAN_MESSAGE,
		"%s %s", chan->channel_name, gnapster_sv());
}

GNAPSTER_COMMAND(do_whois) {
   char *user;
   
   user = next_arg(args, &args);
   if (!user)
     user = user_info.user;
   
   gnapster_text_insert(CURR, SYSTEM,
			"Sending whois request for \2%s\1...\n",
			user);
   napster_send(conn_info.sock, NAPSTER_WHOIS_REQUEST, user);
}

GNAPSTER_COMMAND(do_msg) {
   char *user, *message;
   
   user = next_arg(args, &args);
   message = last_arg(args, &args);
   if (!message)
     INVALID_ARGS();
   
   gnapster_text_insert(CURR, MESSAGE,
			"-> \2[%s] \1%s\n", user, message);
   napster_send(conn_info.sock, NAPSTER_USER_MESSAGE, "%s %s",
		user, message);
}

GNAPSTER_COMMAND(do_me) {
   char *chan;

   CHECK_ARGS();
   
   chan = get_current_chan();
   if (!chan)
     return;
   
   gnapster_text_insert(CURR, SYSTEM, "\2%s\1 %s\n", user_info.user, args);
   napster_send(conn_info.sock, NAPSTER_ACTION, "%s \"%s\"",
		chan, args);
}

GNAPSTER_COMMAND(do_query) {
   if (!args) {
      ChannelInfo *data;
      
      data = g_list_nth_data(console_tab.channel_list,
			     gnapster_get_pagenum() - 2);
      if (!data) return;
      if (data->user)
	remove_console_tab(1);
      else
	gnapster_text_insert(CURR, SYSTEM,
			     "Current tab is not a user query, cannot remove\n");
   } else {
      char *user;
      
      user = next_arg(args, &args);
      if (user_lookup(user) != -1) {
	 gnapster_text_insert(CURR, SYSTEM,
			      "You are already talking to \2%s\1\n",
			      user);
      } else {
	 append_console_tab(user, 1);
	 if (command)
	   gtk_notebook_set_page(GTK_NOTEBOOK(console_tab.notebook),
				 user_lookup(user) - 1);
      }
   }
}

GNAPSTER_COMMAND(do_ignore) {
   if (!args) {
      GList *ptr;
      ptr = ignore_list;
      if (ptr) 
	gnapster_text_insert(CURR, SYSTEM,
			     "Ignore list:\n");
      else
	gnapster_text_insert(CURR, SYSTEM,
			     "You do not have anyone on your ignore list\n");
      while(ptr) {
	 gnapster_text_insert(CURR, MESSAGE, "     \2%s\1\n",
			      ptr->data);
	 ptr = ptr->next;
      }
   } else {
      char *user;
      user = next_arg(args, &args);
      if (user_ignored(user)) {
	 gnapster_text_insert(CURR, SYSTEM,
			      "\2%s\1 is already on ignore\n", user);
      } else if (!strcmp(user, user_info.user)) {
	 gnapster_text_insert(CURR, SYSTEM,
			      "You cannot ignore yourself\n");
      } else {
	 gnapster_text_insert(CURR, SYSTEM,
			      "Now ignoring \2%s\1...\n", user);
	 ignore_list = g_list_append(ignore_list, d_strdup(user));
      }
   }
}

GNAPSTER_COMMAND(do_unignore) {
   GList *link;
   char *user;
   
   CHECK_ARGS();
   
   user = next_arg(args, &args);
   
   link = ignore_list;
   while(link) {
      if (!link->data) 
	break;
      if (!strcmp(link->data, user))
	break;
      link = link->next;
   }
   if (link) {
      gnapster_text_insert(CURR, SYSTEM,
			   "Removing \2%s\1 from the ignore list\n",
			   user);
      ignore_list = g_list_remove_link(ignore_list, link);
      d_free(link->data);
   } else {
      gnapster_text_insert(CURR, SYSTEM,
			   "User \2%s\1 is not in the ignore list\n",
			   user);
   }
}

GNAPSTER_COMMAND(do_join) {
   char *chan;
   
   CHECK_ARGS();
   
   chan = next_arg(args, &args);
   gnapster_text_insert(CURR, SYSTEM,
			"Joining \2%s\1...\n", chan);
   napster_send(conn_info.sock, NAPSTER_JOIN_CHAN, chan);
}

GNAPSTER_COMMAND(do_part) {
   if (!remove_console_tab(0))
     gnapster_text_insert(CURR, SYSTEM,
			  "You are not in a channel tab\n");
}

GNAPSTER_COMMAND(do_raw) {
   int srvmsg;
   char *message; 
   
   CHECK_ARGS();
   
   srvmsg = my_atoi(next_arg(args, &args));
   message = last_arg(args, &args);
   napster_send(conn_info.sock, srvmsg, message);
   gnapster_text_insert(CURR, SYSTEM,
			"Sending raw server message: \2%i %s\1\n",
			srvmsg, message);
}

GNAPSTER_COMMAND(do_kill) {
   char *user, *reason;
   
   user = next_arg(args, &args);
   reason = last_arg(args, &args);
   
   if (!user)
     INVALID_ARGS();
   
   if (reason) 
     napster_send(conn_info.sock, NAPSTER_KILL_USER, "%s \"%s\"",
		  user, reason);
   else
     napster_send(conn_info.sock, NAPSTER_KILL_USER, "%s",
		  user);   
}

GNAPSTER_COMMAND(do_kick) {
   char *chan;
   char *user, *reason;
   
   user = next_arg(args, &args);
   reason = last_arg(args, &args);
   
   if (!user)
     INVALID_ARGS();

   chan = get_current_chan();
   if (!chan)
     return;
   
   if (reason) 
     napster_send(conn_info.sock, NAPSTER_KICK, "%s %s \"%s\"",
		  chan, user, reason);
   else
     napster_send(conn_info.sock, NAPSTER_KICK, "%s %s",
		  chan, user);
}

GNAPSTER_COMMAND(do_ban) {
   char *user, *reason;
   
   CHECK_ARGS();
   
   user = next_arg(args, &args);
   reason = last_arg(args, &args);
   
   if (reason)
     napster_send(conn_info.sock, NAPSTER_BAN_USER, "%s \"%s\"",
		  user, reason);
   else
     napster_send(conn_info.sock, NAPSTER_BAN_USER, "%s",
		  user);
}

GNAPSTER_COMMAND(do_unban) {
   char *user;
   
   CHECK_ARGS();
   
   user = next_arg(args, &args);
   
   napster_send(conn_info.sock, NAPSTER_UNBAN_USER, "%s",
		user);
}

GNAPSTER_COMMAND(do_bans) {
   gnapster_text_insert(CURR, SYSTEM, "Retrieving ban list...\n");
   napster_send(conn_info.sock, NAPSTER_BAN_LIST, NULL);
}

GNAPSTER_COMMAND(do_muzzle) {
   char *user, *reason;
   
   CHECK_ARGS();
   
   user = next_arg(args, &args);
   reason = last_arg(args, &args);
   
   if (reason)
     napster_send(conn_info.sock, NAPSTER_MUZZLE_USER, "%s \"%s\"",
		  user, reason);
   else
     napster_send(conn_info.sock, NAPSTER_MUZZLE_USER, "%s",
		  user);
}

GNAPSTER_COMMAND(do_unmuzzle) {
   char *user;
   
   CHECK_ARGS();
   
   user = next_arg(args, &args);
   
   napster_send(conn_info.sock, NAPSTER_UNMUZZLE_USER, "%s",
		user);
}

GNAPSTER_COMMAND(do_mode) {
   char *chan, *mode;
   
   CHECK_ARGS();
   
   chan = next_arg(args, &args);
   mode = last_arg(args, &args);

   if (mode) 
     napster_send(conn_info.sock, NAPSTER_ChannelInfo_MODE, "%s %s", chan, mode);
   else
     napster_send(conn_info.sock, NAPSTER_ChannelInfo_MODE, "%s", chan);
}

GNAPSTER_COMMAND(do_setport) {
   char *user;
   int port;
   
   user = next_arg(args, &args);
   port = my_atoi(next_arg(args,  &args));
   if (port < 0)
     INVALID_ARGS();
   
   napster_send(conn_info.sock, NAPSTER_REQUEST_PORTCHANGE, "%s %i",
		user, port);
}

GNAPSTER_COMMAND(do_userlevel) {
   char *user, *ul;
   
   user = next_arg(args, &args);
   ul = next_arg(args, &args);
   
   if (!ul)
     INVALID_ARGS();
   
   napster_send(conn_info.sock, NAPSTER_CHANGE_USERLEVEL, "%s %s",
		user, ul);
}

GNAPSTER_COMMAND(do_setline) {
   char *user, *line;
   extern char **conn_table;
   int i;
   
   user = next_arg(args, &args);
   line = next_arg(args, &args);
   
   if (!line)
     INVALID_ARGS();
   
   for(i=0; i<10; i++)
     if (!strcmp(line, conn_table[i]))
       break;
     
   if (i == 11)
     return;
   
   napster_send(conn_info.sock, NAPSTER_SET_LINESPEED, "%s %i",
		user, i);
}

GNAPSTER_COMMAND(do_links) {
   napster_send(conn_info.sock, NAPSTER_LINKS, NULL);
}

GNAPSTER_COMMAND(do_signore) {
   if (!args) {
      GList *ptr;
      int i = 1, page_num;
      
      page_num = gnapster_get_pagenum();
      ptr = signore_list;
      if (!ptr) {
	 gnapster_text_insert(page_num, SYSTEM,
			      "Your server ignore list is empty\n");
	 return;
      }
      gnapster_text_insert(page_num, SYSTEM,
			   "Ignoring server messages matching: \n");
      while(ptr) {
	 gnapster_text_insert(page_num, MESSAGE,
			      "    \2%i\1 %s\n", i++, (char *)ptr->data);
	 ptr = ptr->next;
      }
      gnapster_text_insert(page_num, SYSTEM,
			   "To unsignore use the index number to the left\n");
      return;
   }
   
   signore_list = g_list_append(signore_list, d_strdup(args));
   gnapster_text_insert(CURR, SYSTEM,
			"Added \2%s\1 to your server ignore list...\n",
			args);
}

GNAPSTER_COMMAND(do_unsignore) {
   GList *ptr;
   int n;
   
   CHECK_ARGS();
   
   n = my_atoi(next_arg(args, &args));
   if (n < 0)
     INVALID_ARGS();

   ptr = g_list_nth(signore_list, n - 1);
   if (!ptr)
     return;
   gnapster_text_insert(CURR, SYSTEM,
			"Removing \2%s\1 from your server ignore list\n",
			ptr->data);
   signore_list = g_list_remove_link(signore_list, ptr);
   d_free(ptr->data);
}

GNAPSTER_COMMAND(do_sigsave) {
   GList *ptr;
   char *str;
   int page_num;
   /* 8k should be enough... */
   char buf[4096 * 2];

   page_num = gnapster_get_pagenum();
   
   ptr = signore_list;
   if (!ptr) {
      gnapster_text_insert(page_num, SYSTEM,
			   "Nothing to save!\n");
      return;
   }
   bzero(buf, sizeof(buf));
   while(ptr) {
      str = ptr->data;
      if (!str)
	continue;
      if ((strlen(str) + strlen(buf)) > sizeof(buf))
	break;
      if (buf[0] != 0)
	strcat(buf, "\4");
      strcat(buf, str);
      ptr = ptr->next;
   }
   if (!strlen(buf))
     return;
   
   gnome_config_set_string("/gnapster/Options/signore", buf);
   gnome_config_sync();
}

GNAPSTER_COMMAND(do_share) {
   char *op, *file;
   
   op = next_arg(args, &args);
   file = next_arg(args, &args);
   
   if (!file || !op || (strcmp(op, "add") && strcmp(op, "remove")))
     INVALID_ARGS();
   
   if (!strcmp(op, "add")) {
      FILE *f;
      int fd;
      char *checksum;
      struct stat st;
      
      if (stat(file, &st) < 0 || !S_ISREG(st.st_mode)) {
	 gnapster_text_insert(CONSOLE, SYSTEM, "File \2%s\1 does not exist!\n",
			      file);
	 return;
      }

      fd = open(file, O_RDONLY);

      if (fd < 0)
	return;

      checksum = get_checksum(fd, 300000);

      f = open_shared("a");
      if (!f)
	return;
      
      fprintf(f, "%lu \"%s\" %s %lu 0 0 0\n",
	      st.st_mtime, conv_win(file), checksum, st.st_size);
      fclose(f);

      napster_send(conn_info.sock, NAPSTER_SUBMIT,
		   "\"%s\" %s %lu 0 0 0",
		   conv_win(file), checksum, st.st_size);
      gnapster_text_insert(CONSOLE, SYSTEM, "Submitting \2%s\1...\n",
			   file);

      d_free(checksum);
   } else if (!strcmp(op, "remove")) {
      FILE *f;
      char *match = NULL;
      char buf[1024 * 2], *pl, *ptr, *sfile;
      GList *list = NULL, *lptr;
      
      f = open_shared("r");
      if (!f)
	return;
      
      while(!feof(f)) {
	 fgets(buf, sizeof(buf) - 1, f);
	 if (feof(f)) break;
	 pl = d_strdup(buf);
	 ptr = buf;
	 next_arg(ptr, &ptr);
	 sfile = next_arg(ptr, &ptr);
	 if (!file)
	   continue;
	 if (match || !strstr(sfile, file))
	   list = g_list_append(list, d_strdup(pl));
	 else
	   match = d_strdup(sfile);
      }
      fclose(f);
      
      if (!match) {
	 gnapster_text_insert(CONSOLE, SYSTEM, "\2%s\1 was not found in your shared database\n",
			      file);
	 return;
      }
      
      f = open_shared("w");
      lptr = list;
      while(lptr) {
	 fprintf(f, "%s", (char *)lptr->data);
	 d_free(lptr->data);
	 lptr = lptr->next;
      }
      fclose(f);
      g_list_free(list);

      napster_send(conn_info.sock, NAPSTER_REMOVE_SHARE, "%s",
		   match);
      gnapster_text_insert(CONSOLE, SYSTEM, "File \2%s\1 was removed from your shared database\n",
			   match);
   }
}

GNAPSTER_COMMAND(do_stats) {
   gnapster_text_insert(CURR, SYSTEM, "Sending server stats request...\n");
   napster_send(conn_info.sock, NAPSTER_SERVER_STATS, NULL);
}

GNAPSTER_COMMAND(do_chanlevel) {
   char *chan, *level;
   
   chan = next_arg(args, &args);
   level = next_arg(args, &args);
   
   if (!level)
     INVALID_ARGS();
   
   napster_send(conn_info.sock, NAPSTER_ChannelInfo_LEVEL, "%s %s",
		chan, level);
}

GNAPSTER_COMMAND(do_usermode) {
   CHECK_ARGS();
   
   napster_send(conn_info.sock, NAPSTER_SET_USER_MODE, "%s",
		args);
}

GNAPSTER_COMMAND(do_op) {
   char *chan, *user;
   
   user = next_arg(args, &args);
   if (!user)
     INVALID_ARGS();
   
   chan = get_current_chan();
   if (!chan)
     return;
   
   napster_send(conn_info.sock, NAPSTER_SET_ChannelInfo_OP, "%s %s",
		chan, user);
}

GNAPSTER_COMMAND(do_deop) {
   char *chan, *user;
   
   user = next_arg(args, &args);
   if (!user)
     INVALID_ARGS();

   chan = get_current_chan();
   if (!chan)
     return;
   
   napster_send(conn_info.sock, NAPSTER_REMOVE_ChannelInfo_OP, "%s %s",
		chan, user);
}

GNAPSTER_COMMAND(do_cban) {
   char *chan, *user, *reason;
   
   user = next_arg(args, &args);
   reason = last_arg(args, &args);
   if (!user)
     INVALID_ARGS();
   
   chan = get_current_chan();
   if (!chan)
     return;
   
   if (reason)
     napster_send(conn_info.sock, NAPSTER_CBAN, "%s %s \"%s\"",
		  chan, user, reason);
   else
     napster_send(conn_info.sock, NAPSTER_CBAN, "%s %s",
		  chan, user);
}

GNAPSTER_COMMAND(do_cunban) {
   char *chan, *user;
   
   user = next_arg(args, &args);
   if (!user)
     INVALID_ARGS();
   
   chan = get_current_chan();
   if (!chan)
     return;

   napster_send(conn_info.sock, NAPSTER_CUNBAN, "%s %s",
		chan, user);
}

GNAPSTER_COMMAND(do_cbans) {
   char *chan;
   
   chan = get_current_chan();
   if (!chan)
     return;
   
   gnapster_text_insert(CURR, SYSTEM, "Retrieving channel ban list for \2%s\1\n",
			chan);
   napster_send(conn_info.sock, NAPSTER_CBAN_LIST, "%s",
		chan);
}

GNAPSTER_COMMAND(do_cunban_all) {
   char *chan;
   
   chan = get_current_chan();
   if (!chan)
     return;
   
   napster_send(conn_info.sock, NAPSTER_CBAN_CLEAR, "%s",
		chan);
}
